@@ -29,7 +29,6 @@ EXCLUDE = (
     "cmd/gvmap/make_map.c",
     "cmd/gvmap/make_map.h",
     "cmd/gvmap/power.c",
-    "cmd/gvpr/gvprmain.c",
     "cmd/mingle/minglemain.cpp",
     "cmd/smyrna/arcball.c",
     "cmd/smyrna/arcball.h",
@@ -113,7 +112,6 @@ EXCLUDE = (
     "lib/ast/fmtesc.c",
     "lib/ast/stresc.c",
     "lib/ast/strmatch.c",
-    "lib/cdt/cdt.h",
     "lib/cdt/dtclose.c",
     "lib/cdt/dtdisc.c",
     "lib/cdt/dtextract.c",
@@ -317,7 +315,6 @@ EXCLUDE = (
     "lib/label/rectangle.c",
     "lib/label/split.q.c",
     "lib/label/split.q.h",
-    "lib/label/xlabels.c",
     "lib/label/xlabels.h",
     "lib/mingle/agglomerative_bundling.cpp",
     "lib/mingle/edge_bundling.cpp",
@@ -334,7 +331,6 @@ EXCLUDE = (
     "lib/neatogen/call_tri.h",
     "lib/neatogen/circuit.c",
     "lib/neatogen/closest.c",
-    "lib/neatogen/closest.h",
     "lib/neatogen/compute_hierarchy.c",
     "lib/neatogen/conjgrad.c",
     "lib/neatogen/conjgrad.h",
@@ -346,11 +342,9 @@ EXCLUDE = (
     "lib/neatogen/delaunay.h",
     "lib/neatogen/digcola.h",
     "lib/neatogen/dijkstra.c",
-    "lib/neatogen/dijkstra.h",
     "lib/neatogen/edges.c",
     "lib/neatogen/edges.h",
     "lib/neatogen/embed_graph.c",
-    "lib/neatogen/embed_graph.h",
     "lib/neatogen/fPQ.h",
     "lib/neatogen/geometry.c",
     "lib/neatogen/geometry.h",
@@ -365,8 +359,6 @@ EXCLUDE = (
     "lib/neatogen/matinv.c",
     "lib/neatogen/matrix_ops.c",
     "lib/neatogen/matrix_ops.h",
-    "lib/neatogen/mem.h",
-    "lib/neatogen/memory.c",
     "lib/neatogen/multispline.c",
     "lib/neatogen/multispline.h",
     "lib/neatogen/neato.h",
@@ -377,7 +369,6 @@ EXCLUDE = (
     "lib/neatogen/overlap.c",
     "lib/neatogen/overlap.h",
     "lib/neatogen/pca.c",
-    "lib/neatogen/pca.h",
     "lib/neatogen/poly.c",
     "lib/neatogen/poly.h",
     "lib/neatogen/quad_prog_solve.c",
@@ -388,7 +379,6 @@ EXCLUDE = (
     "lib/neatogen/randomkit.h",
     "lib/neatogen/sgd.h",
     "lib/neatogen/site.c",
-    "lib/neatogen/site.h",
     "lib/neatogen/smart_ini_x.c",
     "lib/neatogen/sparsegraph.h",
     "lib/neatogen/stress.c",
@@ -469,13 +459,6 @@ EXCLUDE = (
     "lib/twopigen/circle.c",
     "lib/twopigen/circle.h",
     "lib/twopigen/twopiinit.c",
-    "lib/vmalloc/test.c",
-    "lib/vmalloc/vmalloc.c",
-    "lib/vmalloc/vmalloc.h",
-    "lib/vmalloc/vmclear.c",
-    "lib/vmalloc/vmclose.c",
-    "lib/vmalloc/vmopen.c",
-    "lib/vmalloc/vmstrdup.c",
     "lib/vpsc/block.cpp",
     "lib/vpsc/block.h",
     "lib/vpsc/blocks.cpp",
