@@ -18,6 +18,7 @@
 #include <math.h>
 #include <stdatomic.h>
 #include <stdbool.h>
+#include <stddef.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -30,8 +31,7 @@
 #include <ortho/ortho.h>
 #endif
 
-#define NSUB 9    /* number of subdivisions, re-aiming splines */
-#define CHUNK 128 /* in building list of edges */
+#define NSUB 9 /* number of subdivisions, re-aiming splines */
 
 #define MINW 16 /* minimum width of a box in the edge path */
 #define HALFMINW 8
@@ -43,22 +43,21 @@
 #define AUXGRAPH 128
 #define GRAPHTYPEMASK 192 /* the OR of the above */
 
-#define MAKEFWDEDGE(new, old)                                                  \
-  {                                                                            \
-    edge_t *newp;                                                              \
-    Agedgeinfo_t *info;                                                        \
-    newp = new;                                                                \
-    info = (Agedgeinfo_t *)newp->base.data;                                    \
-    *info = *(Agedgeinfo_t *)old->base.data;                                   \
-    *newp = *old;                                                              \
-    newp->base.data = (Agrec_t *)info;                                         \
-    AGTAIL(newp) = AGHEAD(old);                                                \
-    AGHEAD(newp) = AGTAIL(old);                                                \
-    ED_tail_port(newp) = ED_head_port(old);                                    \
-    ED_head_port(newp) = ED_tail_port(old);                                    \
-    ED_edge_type(newp) = VIRTUAL;                                              \
-    ED_to_orig(newp) = old;                                                    \
-  }
+static void makefwdedge(edge_t *new, edge_t *old) {
+  Agedgeinfo_t *const info =
+      (Agedgeinfo_t *)((uintptr_t)new->base.data - offsetof(Agedgeinfo_t, hdr));
+  const Agedgeinfo_t *const old_info =
+      (Agedgeinfo_t *)((uintptr_t)old->base.data - offsetof(Agedgeinfo_t, hdr));
+  *info = *old_info;
+  *new = *old;
+  new->base.data = &info->hdr;
+  AGTAIL(new) = AGHEAD(old);
+  AGHEAD(new) = AGTAIL(old);
+  ED_tail_port(new) = ED_head_port(old);
+  ED_head_port(new) = ED_tail_port(old);
+  ED_edge_type(new) = VIRTUAL;
+  ED_to_orig(new) = old;
+}
 
 typedef struct {
   double LeftBound;
@@ -68,7 +67,7 @@ typedef struct {
   boxf *Rank_box;
 } spline_info_t;
 
-DEFINE_LIST(points, pointf)
+typedef LIST(pointf) points_t;
 
 static void adjustregularpath(path *, size_t, size_t);
 static Agedge_t *bot_bound(Agedge_t *, int);
@@ -78,13 +77,13 @@ static bool cl_vninside(Agraph_t *, Agnode_t *);
 static void completeregularpath(path *, Agedge_t *, Agedge_t *, pathend_t *,
                                 pathend_t *, const boxes_t *);
 static int edgecmp(const void *, const void *);
-static int make_flat_edge(graph_t *, spline_info_t *, path *, Agedge_t **,
+static int make_flat_edge(graph_t *, const spline_info_t, path *, Agedge_t **,
                           unsigned, unsigned, int);
 static void make_regular_edge(graph_t *g, spline_info_t *, path *, Agedge_t **,
                               unsigned, unsigned, int);
 static boxf makeregularend(boxf, int, double);
-static boxf maximal_bbox(graph_t *g, spline_info_t *, Agnode_t *, Agedge_t *,
-                         Agedge_t *);
+static boxf maximal_bbox(graph_t *g, const spline_info_t, Agnode_t *,
+                         Agedge_t *, Agedge_t *);
 static Agnode_t *neighbor(graph_t *, Agnode_t *, Agedge_t *, Agedge_t *, int);
 static void place_vnlabel(Agnode_t *);
 static boxf rank_box(spline_info_t *sp, Agraph_t *, int);
@@ -95,11 +94,6 @@ static int straight_len(Agnode_t *);
 static Agedge_t *straight_path(Agedge_t *, int, points_t *);
 static Agedge_t *top_bound(Agedge_t *, int);
 
-#define GROWEDGES                                                              \
-  do {                                                                         \
-    edges = gv_recalloc(edges, n_edges, n_edges + CHUNK, sizeof(edge_t *));    \
-  } while (0)
-
 static edge_t *getmainedge(edge_t *e) {
   edge_t *le = e;
   while (ED_to_virt(le))
@@ -169,8 +163,7 @@ static void swap_spline(splines *s) {
   }
 }
 
-/* edge_normalize:
- * Some back edges are reversed during layout and the reversed edge
+/* Some back edges are reversed during layout and the reversed edge
  * is used to compute the spline. We would like to guarantee that
  * the order of control points always goes from tail to head, so
  * we reverse them if necessary.
@@ -187,8 +180,7 @@ static void edge_normalize(graph_t *g) {
   }
 }
 
-/* resetRW:
- * In position, each node has its rw stored in mval and,
+/* In position, each node has its rw stored in mval and,
  * if a node is part of a loop, rw may be increased to
  * reflect the loops and associated labels. We restore
  * the original value here.
@@ -203,8 +195,7 @@ static void resetRW(graph_t *g) {
   }
 }
 
-/* setEdgeLabelPos:
- * Set edge label position information for regular and non-adjacent flat edges.
+/* Set edge label position information for regular and non-adjacent flat edges.
  * Dot has allocated space and position for these labels. This info will be
  * used when routing orthogonal edges.
  */
@@ -239,15 +230,15 @@ static void setEdgeLabelPos(graph_t *g) {
  * @return 0 on success
  */
 static int dot_splines_(graph_t *g, int normalize) {
-  int i, j, k, n_nodes;
+  int i, j, n_nodes;
   node_t *n;
   Agedgeinfo_t fwdedgeai, fwdedgebi;
   Agedgepair_t fwdedgea, fwdedgeb;
-  edge_t *e, *e0, *e1, *ea, *eb, *le0, *le1, **edges = NULL;
+  edge_t *e, *e0, *e1, *ea, *eb, *le0, *le1;
   path P = {0};
   int et = EDGE_TYPE(g);
-  fwdedgea.out.base.data = (Agrec_t *)&fwdedgeai;
-  fwdedgeb.out.base.data = (Agrec_t *)&fwdedgebi;
+  fwdedgea.out.base.data = &fwdedgeai.hdr;
+  fwdedgeb.out.base.data = &fwdedgebi.hdr;
 
   if (et == EDGETYPE_NONE)
     return 0;
@@ -258,6 +249,8 @@ static int dot_splines_(graph_t *g, int normalize) {
                  "xlabels\n");
     }
   }
+  spline_info_t sd = {0};
+  LIST(edge_t *) edges = {0};
 #ifdef ORTHO
   if (et == EDGETYPE_ORTHO) {
     resetRW(g);
@@ -275,19 +268,17 @@ static int dot_splines_(graph_t *g, int normalize) {
   mark_lowclusters(g);
   if (routesplinesinit())
     return 0;
-  spline_info_t sd = {.Splinesep = GD_nodesep(g) / 4,
-                      .Multisep = GD_nodesep(g)};
-  edges = gv_calloc(CHUNK, sizeof(edge_t *));
+  sd = (spline_info_t){.Splinesep = GD_nodesep(g) / 4,
+                       .Multisep = GD_nodesep(g)};
 
   /* compute boundaries and list of splines */
-  unsigned n_edges = 0;
   n_nodes = 0;
   for (i = GD_minrank(g); i <= GD_maxrank(g); i++) {
     n_nodes += GD_rank(g)[i].n;
     if ((n = GD_rank(g)[i].v[0]))
-      sd.LeftBound = MIN(sd.LeftBound, (ND_coord(n).x - ND_lw(n)));
+      sd.LeftBound = MIN(sd.LeftBound, ND_coord(n).x - ND_lw(n));
     if (GD_rank(g)[i].n && (n = GD_rank(g)[i].v[GD_rank(g)[i].n - 1]))
-      sd.RightBound = MAX(sd.RightBound, (ND_coord(n).x + ND_rw(n)));
+      sd.RightBound = MAX(sd.RightBound, ND_coord(n).x + ND_rw(n));
     sd.LeftBound -= MINW;
     sd.RightBound += MINW;
 
@@ -304,20 +295,16 @@ static int dot_splines_(graph_t *g, int normalize) {
       }
       if (ND_node_type(n) != NORMAL && !sinfo.splineMerge(n))
         continue;
-      for (k = 0; (e = ND_out(n).list[k]); k++) {
+      for (int k = 0; (e = ND_out(n).list[k]); k++) {
         if (ED_edge_type(e) == FLATORDER || ED_edge_type(e) == IGNORED)
           continue;
         setflags(e, REGULAREDGE, FWDEDGE, MAINGRAPH);
-        edges[n_edges++] = e;
-        if (n_edges % CHUNK == 0)
-          GROWEDGES;
+        LIST_APPEND(&edges, e);
       }
       if (ND_flat_out(n).list)
-        for (k = 0; (e = ND_flat_out(n).list[k]); k++) {
+        for (int k = 0; (e = ND_flat_out(n).list[k]); k++) {
           setflags(e, FLATEDGE, 0, AUXGRAPH);
-          edges[n_edges++] = e;
-          if (n_edges % CHUNK == 0)
-            GROWEDGES;
+          LIST_APPEND(&edges, e);
         }
       if (ND_other(n).list) {
         /* In position, each node has its rw stored in mval and,
@@ -328,11 +315,9 @@ static int dot_splines_(graph_t *g, int normalize) {
         if (ND_node_type(n) == NORMAL) {
           SWAP(&ND_rw(n), &ND_mval(n));
         }
-        for (k = 0; (e = ND_other(n).list[k]); k++) {
+        for (int k = 0; (e = ND_other(n).list[k]); k++) {
           setflags(e, 0, 0, AUXGRAPH);
-          edges[n_edges++] = e;
-          if (n_edges % CHUNK == 0)
-            GROWEDGES;
+          LIST_APPEND(&edges, e);
         }
       }
     }
@@ -344,7 +329,7 @@ static int dot_splines_(graph_t *g, int normalize) {
    * alternatively, the edges would be routed identically if
    * routed separately.
    */
-  qsort(edges, n_edges, sizeof(edges[0]), edgecmp);
+  LIST_SORT(&edges, edgecmp);
 
   /* FIXME: just how many boxes can there be? */
   P.boxes = gv_calloc(n_nodes + 20 * 2 * NSUB, sizeof(boxf));
@@ -359,21 +344,21 @@ static int dot_splines_(graph_t *g, int normalize) {
     }
   }
 
-  for (unsigned l = 0; l < n_edges;) {
+  for (unsigned l = 0; l < LIST_SIZE(&edges);) {
     const unsigned ind = l;
-    le0 = getmainedge((e0 = edges[l++]));
+    le0 = getmainedge((e0 = LIST_GET(&edges, l++)));
     if (ED_tail_port(e0).defined || ED_head_port(e0).defined) {
       ea = e0;
     } else {
       ea = le0;
     }
     if (ED_tree_index(ea) & BWDEDGE) {
-      MAKEFWDEDGE(&fwdedgea.out, ea);
+      makefwdedge(&fwdedgea.out, ea);
       ea = &fwdedgea.out;
     }
     unsigned cnt;
-    for (cnt = 1; l < n_edges; cnt++, l++) {
-      if (le0 != (le1 = getmainedge((e1 = edges[l]))))
+    for (cnt = 1; l < LIST_SIZE(&edges); cnt++, l++) {
+      if (le0 != (le1 = getmainedge((e1 = LIST_GET(&edges, l)))))
         break;
       if (ED_adjacent(e0))
         continue; /* all flat adjacent edges at once */
@@ -383,7 +368,7 @@ static int dot_splines_(graph_t *g, int normalize) {
         eb = le1;
       }
       if (ED_tree_index(eb) & BWDEDGE) {
-        MAKEFWDEDGE(&fwdedgeb.out, eb);
+        makefwdedge(&fwdedgeb.out, eb);
         eb = &fwdedgeb.out;
       }
       if (portcmp(ED_tail_port(ea), ED_tail_port(eb)))
@@ -393,22 +378,21 @@ static int dot_splines_(graph_t *g, int normalize) {
       if ((ED_tree_index(e0) & EDGETYPEMASK) == FLATEDGE &&
           ED_label(e0) != ED_label(e1))
         break;
-      if (ED_tree_index(edges[l]) & MAINGRAPH) /* Aha! -C is on */
+      if (ED_tree_index(LIST_GET(&edges, l)) & MAINGRAPH) /* Aha! -C is on */
         break;
     }
 
     if (et == EDGETYPE_CURVED) {
       edge_t **edgelist = gv_calloc(cnt, sizeof(edge_t *));
-      edgelist[0] = getmainedge((edges + ind)[0]);
+      edgelist[0] = getmainedge(LIST_GET(&edges, ind));
       for (unsigned ii = 1; ii < cnt; ii++)
-        edgelist[ii] = (edges + ind)[ii];
+        edgelist[ii] = LIST_GET(&edges, ind + ii);
       makeStraightEdges(g, edgelist, cnt, et, &sinfo);
       free(edgelist);
     } else if (agtail(e0) == aghead(e0)) {
-      int r;
       double sizey;
       n = agtail(e0);
-      r = ND_rank(n);
+      const int r = ND_rank(n);
       if (r == GD_maxrank(g)) {
         if (r > 0)
           sizey = ND_coord(GD_rank(g)[r - 1].v[0]).y - ND_coord(n).y;
@@ -421,19 +405,23 @@ static int dot_splines_(graph_t *g, int normalize) {
         double dwny = ND_coord(n).y - ND_coord(GD_rank(g)[r + 1].v[0]).y;
         sizey = MIN(upy, dwny);
       }
-      makeSelfEdge(edges, ind, cnt, sd.Multisep, sizey / 2, &sinfo);
+      makeSelfEdge(LIST_FRONT(&edges), ind, cnt, sd.Multisep, sizey / 2,
+                   &sinfo);
       for (unsigned b = 0; b < cnt; b++) {
-        e = edges[ind + b];
+        e = LIST_GET(&edges, ind + b);
         if (ED_label(e))
           updateBB(g, ED_label(e));
       }
     } else if (ND_rank(agtail(e0)) == ND_rank(aghead(e0))) {
-      const int rc = make_flat_edge(g, &sd, &P, edges, ind, cnt, et);
+      const int rc =
+          make_flat_edge(g, sd, &P, LIST_FRONT(&edges), ind, cnt, et);
       if (rc != 0) {
+        free(sd.Rank_box);
+        LIST_FREE(&edges);
         return rc;
       }
     } else
-      make_regular_edge(g, &sd, &P, edges, ind, cnt, et);
+      make_regular_edge(g, &sd, &P, LIST_FRONT(&edges), ind, cnt, et);
   }
 
   /* place regular edge labels */
@@ -479,25 +467,23 @@ finish:
 #else
   if (et != EDGETYPE_CURVED) {
 #endif
-    free(sd.Rank_box);
     routesplinesterm();
   }
-  free(edges);
+  free(sd.Rank_box);
+  LIST_FREE(&edges);
   free(P.boxes);
   State = GVSPLINES;
   EdgeLabelsDone = 1;
   return 0;
 }
 
-/* dot_splines:
- * If the splines attribute is defined but equal to "", skip edge routing.
+/* If the splines attribute is defined but equal to "", skip edge routing.
  *
  * @return 0 on success
  */
 int dot_splines(graph_t *g) { return dot_splines_(g, 1); }
 
-/* place_vnlabel:
- * assign position of an edge label from its virtual node
+/* assign position of an edge label from its virtual node
  * This is for regular edges only.
  */
 static void place_vnlabel(node_t *n) {
@@ -540,11 +526,10 @@ static void setflags(edge_t *e, int hint1, int hint2, int f3) {
     else /* f1 == SELF*EDGE */
       f2 = FWDEDGE;
   }
-  ED_tree_index(e) = (f1 | f2 | f3);
+  ED_tree_index(e) = f1 | f2 | f3;
 }
 
-/* edgecmp:
- * lexicographically order edges by
+/* lexicographically order edges by
  *  - edge type
  *  - |rank difference of nodes|
  *  - |x difference of nodes|
@@ -554,29 +539,18 @@ static void setflags(edge_t *e, int hint1, int hint2, int f3) {
  *  - labels if flat edges
  *  - edge id
  */
-static int edgecmp(const void *x, const void *y) {
-// Suppress Clang/GCC -Wcast-qual warning. Casting away const here is acceptable
-// as the later usage is const. We need the cast because the macros use
-// non-const pointers for genericity.
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wcast-qual"
-#endif
-  edge_t **ptr0 = (edge_t **)x;
-  edge_t **ptr1 = (edge_t **)y;
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif
+static int edgecmp(const void *p0, const void *p1) {
+  edge_t *const *ptr0 = p0;
+  edge_t *const *ptr1 = p1;
   Agedgeinfo_t fwdedgeai, fwdedgebi;
   Agedgepair_t fwdedgea, fwdedgeb;
-  edge_t *e0, *e1, *ea, *eb, *le0, *le1;
-  int et0, et1, v0, v1, rv;
-  double t0, t1;
+  edge_t *ea, *eb, *le0, *le1;
+  int et0, et1, rv;
 
-  fwdedgea.out.base.data = (Agrec_t *)&fwdedgeai;
-  fwdedgeb.out.base.data = (Agrec_t *)&fwdedgebi;
-  e0 = *ptr0;
-  e1 = *ptr1;
+  fwdedgea.out.base.data = &fwdedgeai.hdr;
+  fwdedgeb.out.base.data = &fwdedgebi.hdr;
+  edge_t *const e0 = *ptr0;
+  edge_t *const e1 = *ptr1;
   et0 = ED_tree_index(e0) & EDGETYPEMASK;
   et1 = ED_tree_index(e1) & EDGETYPEMASK;
   if (et0 < et1) {
@@ -589,27 +563,30 @@ static int edgecmp(const void *x, const void *y) {
   le0 = getmainedge(e0);
   le1 = getmainedge(e1);
 
-  t0 = ND_rank(agtail(le0)) - ND_rank(aghead(le0));
-  t1 = ND_rank(agtail(le1)) - ND_rank(aghead(le1));
-  v0 = abs((int)t0); /* ugly, but explicit as to how we avoid equality tests on
-                        fp numbers */
-  v1 = abs((int)t1);
-  if (v0 < v1) {
-    return -1;
-  }
-  if (v0 > v1) {
-    return 1;
+  {
+    const int rank_diff0 = ND_rank(agtail(le0)) - ND_rank(aghead(le0));
+    const int rank_diff1 = ND_rank(agtail(le1)) - ND_rank(aghead(le1));
+    const int v0 = abs(rank_diff0);
+    const int v1 = abs(rank_diff1);
+    if (v0 < v1) {
+      return -1;
+    }
+    if (v0 > v1) {
+      return 1;
+    }
   }
 
-  t0 = ND_coord(agtail(le0)).x - ND_coord(aghead(le0)).x;
-  t1 = ND_coord(agtail(le1)).x - ND_coord(aghead(le1)).x;
-  v0 = abs((int)t0);
-  v1 = abs((int)t1);
-  if (v0 < v1) {
-    return -1;
-  }
-  if (v0 > v1) {
-    return 1;
+  {
+    const double t0 = ND_coord(agtail(le0)).x - ND_coord(aghead(le0)).x;
+    const double t1 = ND_coord(agtail(le1)).x - ND_coord(aghead(le1)).x;
+    const double v0 = fabs(t0);
+    const double v1 = fabs(t1);
+    if (v0 < v1) {
+      return -1;
+    }
+    if (v0 > v1) {
+      return 1;
+    }
   }
 
   /* This provides a cheap test for edges having the same set of endpoints.  */
@@ -622,12 +599,12 @@ static int edgecmp(const void *x, const void *y) {
 
   ea = (ED_tail_port(e0).defined || ED_head_port(e0).defined) ? e0 : le0;
   if (ED_tree_index(ea) & BWDEDGE) {
-    MAKEFWDEDGE(&fwdedgea.out, ea);
+    makefwdedge(&fwdedgea.out, ea);
     ea = &fwdedgea.out;
   }
   eb = (ED_tail_port(e1).defined || ED_head_port(e1).defined) ? e1 : le1;
   if (ED_tree_index(eb) & BWDEDGE) {
-    MAKEFWDEDGE(&fwdedgeb.out, eb);
+    makefwdedge(&fwdedgeb.out, eb);
     eb = &fwdedgeb.out;
   }
   if ((rv = portcmp(ED_tail_port(ea), ED_tail_port(eb))))
@@ -793,8 +770,7 @@ static void setState(graph_t *auxg, attr_state_t *attr_state) {
   G_ordering = agfindgraphattr(auxg, "ordering");
 }
 
-/* cloneGraph:
- * Create clone graph. It stores the global Agsyms, to be
+/* Create clone graph. It stores the global Agsyms, to be
  * restored in cleanupCloneGraph. The graph uses the main
  * graph's settings for certain geometry parameters, and
  * declares all node and edge attributes used in the original
@@ -894,8 +870,7 @@ static void cleanupCloneGraph(graph_t *g, attr_state_t *attr_state) {
   agclose(g);
 }
 
-/* cloneNode:
- * If original graph has rankdir=LR or RL, records change shape,
+/* If original graph has rankdir=LR or RL, records change shape,
  * so we wrap a record node's label in "{...}" to prevent this.
  */
 static node_t *cloneNode(graph_t *g, node_t *orign) {
@@ -920,9 +895,7 @@ static edge_t *cloneEdge(graph_t *g, node_t *tn, node_t *hn, edge_t *orig) {
   return e;
 }
 
-/* transformf:
- * Rotate, if necessary, then translate points.
- */
+/// rotate, if necessary, then translate points
 static pointf transformf(pointf p, pointf del, int flip) {
   if (flip) {
     double i = p.x;
@@ -932,8 +905,7 @@ static pointf transformf(pointf p, pointf del, int flip) {
   return add_pointf(p, del);
 }
 
-/* edgelblcmpfn:
- * lexicographically order edges by
+/* lexicographically order edges by
  *  - has label
  *  - label is wider
  *  - label is higher
@@ -980,8 +952,7 @@ static int edgelblcmpfn(const void *x, const void *y) {
 
 #define LBL_SPACE 6 /* space between labels, in points */
 
-/* makeSimpleFlatLabels:
- * This handles the second simplest case for flat edges between
+/* This handles the second simplest case for flat edges between
  * two adjacent nodes. We still invoke a dot on a rotated problem
  * to handle edges with ports. This usually works, but fails for
  * records because of their weird nature.
@@ -1181,8 +1152,7 @@ static void makeSimpleFlat(node_t *tn, node_t *hn, edge_t **edges, unsigned ind,
   }
 }
 
-/* make_flat_adj_edges:
- * In the simple case, with no labels or ports, this creates a simple
+/* In the simple case, with no labels or ports, this creates a simple
  * spindle of splines.
  * If there are only labels, cobble something together.
  * Otherwise, we run dot recursively on the 2 nodes and the edges,
@@ -1348,11 +1318,9 @@ static int make_flat_adj_edges(graph_t *g, edge_t **edges, unsigned ind,
   return 0;
 }
 
-static void makeFlatEnd(graph_t *g, spline_info_t *sp, path *P, node_t *n,
+static void makeFlatEnd(graph_t *g, const spline_info_t sp, path *P, node_t *n,
                         edge_t *e, pathend_t *endp, bool isBegin) {
-  boxf b;
-
-  b = endp->nb = maximal_bbox(g, sp, n, NULL, e);
+  boxf b = endp->nb = maximal_bbox(g, sp, n, NULL, e);
   endp->sidemask = TOP;
   if (isBegin)
     beginpath(P, e, FLATEDGE, endp, false);
@@ -1365,11 +1333,10 @@ static void makeFlatEnd(graph_t *g, spline_info_t *sp, path *P, node_t *n,
     endp->boxes[endp->boxn++] = b;
 }
 
-static void makeBottomFlatEnd(graph_t *g, spline_info_t *sp, path *P, node_t *n,
-                              edge_t *e, pathend_t *endp, bool isBegin) {
-  boxf b;
-
-  b = endp->nb = maximal_bbox(g, sp, n, NULL, e);
+static void makeBottomFlatEnd(graph_t *g, const spline_info_t sp, path *P,
+                              node_t *n, edge_t *e, pathend_t *endp,
+                              bool isBegin) {
+  boxf b = endp->nb = maximal_bbox(g, sp, n, NULL, e);
   endp->sidemask = BOTTOM;
   if (isBegin)
     beginpath(P, e, FLATEDGE, endp, false);
@@ -1382,7 +1349,7 @@ static void makeBottomFlatEnd(graph_t *g, spline_info_t *sp, path *P, node_t *n,
     endp->boxes[endp->boxn++] = b;
 }
 
-static void make_flat_labeled_edge(graph_t *g, spline_info_t *sp, path *P,
+static void make_flat_labeled_edge(graph_t *g, const spline_info_t sp, path *P,
                                    edge_t *e, int et) {
   node_t *tn, *hn, *ln;
   pointf *ps;
@@ -1486,7 +1453,7 @@ static void make_flat_labeled_edge(graph_t *g, spline_info_t *sp, path *P,
     free(ps);
 }
 
-static void make_flat_bottom_edges(graph_t *g, spline_info_t *sp, path *P,
+static void make_flat_bottom_edges(graph_t *g, const spline_info_t sp, path *P,
                                    edge_t **edges, unsigned ind, unsigned cnt,
                                    edge_t *e, bool use_splines) {
   node_t *tn, *hn;
@@ -1505,7 +1472,7 @@ static void make_flat_bottom_edges(graph_t *g, spline_info_t *sp, path *P,
   } else {
     vspace = GD_ranksep(g);
   }
-  stepx = sp->Multisep / (cnt + 1);
+  stepx = sp.Multisep / (cnt + 1);
   stepy = vspace / (cnt + 1);
 
   makeBottomFlatEnd(g, sp, P, tn, e, &tend, true);
@@ -1560,8 +1527,7 @@ static void make_flat_bottom_edges(graph_t *g, spline_info_t *sp, path *P,
   }
 }
 
-/* make_flat_edge:
- * Construct flat edges edges[ind...ind+cnt-1]
+/* Construct flat edges edges[ind...ind+cnt-1]
  * There are 4 main cases:
  *  - all edges between a and b where a and b are adjacent
  *  - one labeled edge
@@ -1571,7 +1537,7 @@ static void make_flat_bottom_edges(graph_t *g, spline_info_t *sp, path *P,
  *
  * @return 0 on success
  */
-static int make_flat_edge(graph_t *g, spline_info_t *sp, path *P,
+static int make_flat_edge(graph_t *g, const spline_info_t sp, path *P,
                           edge_t **edges, unsigned ind, unsigned cnt, int et) {
   node_t *tn, *hn;
   Agedgeinfo_t fwdedgei;
@@ -1582,13 +1548,13 @@ static int make_flat_edge(graph_t *g, spline_info_t *sp, path *P,
   int tside, hside;
   pathend_t tend, hend;
 
-  fwdedge.out.base.data = (Agrec_t *)&fwdedgei;
+  fwdedge.out.base.data = &fwdedgei.hdr;
 
   /* Get sample edge; normalize to go from left to right */
   e = edges[ind];
   bool isAdjacent = ED_adjacent(e) != 0;
   if (ED_tree_index(e) & BWDEDGE) {
-    MAKEFWDEDGE(&fwdedge.out, e);
+    makefwdedge(&fwdedge.out, e);
     e = &fwdedge.out;
   }
   for (unsigned i = 1; i < cnt; i++) {
@@ -1634,7 +1600,7 @@ static int make_flat_edge(graph_t *g, spline_info_t *sp, path *P,
   } else {
     vspace = GD_ranksep(g);
   }
-  stepx = sp->Multisep / (cnt + 1);
+  stepx = sp.Multisep / (cnt + 1);
   stepy = vspace / (cnt + 1);
 
   makeFlatEnd(g, sp, P, tn, e, &tend, true);
@@ -1695,8 +1661,7 @@ static bool leftOf(pointf p1, pointf p2, pointf p3) {
   return (p1.y - p2.y) * (p3.x - p2.x) - (p3.y - p2.y) * (p1.x - p2.x) > 0;
 }
 
-/* makeLineEdge:
- * Create an edge as line segment. We guarantee that the points
+/* Create an edge as line segment. We guarantee that the points
  * are always drawn downwards. This means that for flipped edges,
  * we draw from the head to the tail. The routine returns the
  * end node of the edge in *hp. The points are stored in the
@@ -1755,19 +1720,19 @@ static int makeLineEdge(graph_t *g, edge_t *fe, points_t *points, node_t **hp) {
       lp.y += height / 2.0;
     }
 
-    points_append(points, startp);
-    points_append(points, startp);
-    points_append(points, lp);
-    points_append(points, lp);
-    points_append(points, lp);
-    points_append(points, endp);
-    points_append(points, endp);
+    LIST_APPEND(points, startp);
+    LIST_APPEND(points, startp);
+    LIST_APPEND(points, lp);
+    LIST_APPEND(points, lp);
+    LIST_APPEND(points, lp);
+    LIST_APPEND(points, endp);
+    LIST_APPEND(points, endp);
     pn = 7;
   } else {
-    points_append(points, startp);
-    points_append(points, startp);
-    points_append(points, endp);
-    points_append(points, endp);
+    LIST_APPEND(points, startp);
+    LIST_APPEND(points, startp);
+    LIST_APPEND(points, endp);
+    LIST_APPEND(points, endp);
     pn = 4;
   }
 
@@ -1787,26 +1752,28 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
   points_t pointfs = {0};
   points_t pointfs2 = {0};
 
-  fwdedgea.out.base.data = (Agrec_t *)&fwdedgeai;
-  fwdedgeb.out.base.data = (Agrec_t *)&fwdedgebi;
-  fwdedge.out.base.data = (Agrec_t *)&fwdedgei;
+  fwdedgea.out.base.data = &fwdedgeai.hdr;
+  fwdedgeb.out.base.data = &fwdedgebi.hdr;
+  fwdedge.out.base.data = &fwdedgei.hdr;
 
   sl = 0;
   e = edges[ind];
   bool hackflag = false;
   if (abs(ND_rank(agtail(e)) - ND_rank(aghead(e))) > 1) {
-    fwdedgeai = *(Agedgeinfo_t *)e->base.data;
+    fwdedgeai = *(Agedgeinfo_t *)((uintptr_t)e->base.data -
+                                  offsetof(Agedgeinfo_t, hdr));
     fwdedgea.out = *e;
     fwdedgea.in = *AGOUT2IN(e);
-    fwdedgea.out.base.data = (Agrec_t *)&fwdedgeai;
+    fwdedgea.out.base.data = &fwdedgeai.hdr;
     if (ED_tree_index(e) & BWDEDGE) {
-      MAKEFWDEDGE(&fwdedgeb.out, e);
+      makefwdedge(&fwdedgeb.out, e);
       agtail(&fwdedgea.out) = aghead(e);
       ED_tail_port(&fwdedgea.out) = ED_head_port(e);
     } else {
-      fwdedgebi = *(Agedgeinfo_t *)e->base.data;
+      fwdedgebi = *(Agedgeinfo_t *)((uintptr_t)e->base.data -
+                                    offsetof(Agedgeinfo_t, hdr));
       fwdedgeb.out = *e;
-      fwdedgeb.out.base.data = (Agrec_t *)&fwdedgebi;
+      fwdedgeb.out.base.data = &fwdedgebi.hdr;
       agtail(&fwdedgea.out) = agtail(e);
       fwdedgeb.in = *AGOUT2IN(e);
     }
@@ -1822,7 +1789,7 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
     hackflag = true;
   } else {
     if (ED_tree_index(e) & BWDEDGE) {
-      MAKEFWDEDGE(&fwdedgea.out, e);
+      makefwdedge(&fwdedgea.out, e);
       e = &fwdedgea.out;
     }
   }
@@ -1837,7 +1804,7 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
     segfirst = e;
     tn = agtail(e);
     hn = aghead(e);
-    b = tend.nb = maximal_bbox(g, sp, tn, NULL, e);
+    b = tend.nb = maximal_bbox(g, *sp, tn, NULL, e);
     beginpath(P, e, REGULAREDGE, &tend, spline_merge(tn));
     b.UR.y = tend.boxes[tend.boxn - 1].UR.y;
     b.LL.y = tend.boxes[tend.boxn - 1].LL.y;
@@ -1847,7 +1814,7 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
     bool smode = false;
     si = -1;
     while (ND_node_type(hn) == VIRTUAL && !sinfo.splineMerge(hn)) {
-      boxes_append(&boxes, rank_box(sp, g, ND_rank(tn)));
+      LIST_APPEND(&boxes, rank_box(sp, g, ND_rank(tn)));
       if (!smode && ((sl = straight_len(hn)) >=
                      ((GD_has_labels(g->root) & EDGE_LABEL) ? 4 + 1 : 2 + 1))) {
         smode = true;
@@ -1855,13 +1822,13 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
       }
       if (!smode || si > 0) {
         si--;
-        boxes_append(&boxes, maximal_bbox(g, sp, hn, e, ND_out(hn).list[0]));
+        LIST_APPEND(&boxes, maximal_bbox(g, *sp, hn, e, ND_out(hn).list[0]));
         e = ND_out(hn).list[0];
         tn = agtail(e);
         hn = aghead(e);
         continue;
       }
-      hend.nb = maximal_bbox(g, sp, hn, e, ND_out(hn).list[0]);
+      hend.nb = maximal_bbox(g, *sp, hn, e, ND_out(hn).list[0]);
       endpath(P, e, REGULAREDGE, &hend, spline_merge(aghead(e)));
       b = makeregularend(hend.boxes[hend.boxn - 1], TOP,
                          ND_coord(hn).y + GD_rank(g)[ND_rank(hn)].ht2);
@@ -1883,14 +1850,14 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
       }
       if (pn == 0) {
         free(ps);
-        boxes_free(&boxes);
-        points_free(&pointfs);
-        points_free(&pointfs2);
+        LIST_FREE(&boxes);
+        LIST_FREE(&pointfs);
+        LIST_FREE(&pointfs2);
         return;
       }
 
       for (size_t i = 0; i < pn; i++) {
-        points_append(&pointfs, ps[i]);
+        LIST_APPEND(&pointfs, ps[i]);
       }
       free(ps);
       e = straight_path(ND_out(hn).list[0], sl, &pointfs);
@@ -1898,8 +1865,8 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
       segfirst = e;
       tn = agtail(e);
       hn = aghead(e);
-      boxes_clear(&boxes);
-      tend.nb = maximal_bbox(g, sp, tn, ND_in(tn).list[0], e);
+      LIST_CLEAR(&boxes);
+      tend.nb = maximal_bbox(g, *sp, tn, ND_in(tn).list[0], e);
       beginpath(P, e, REGULAREDGE, &tend, spline_merge(tn));
       b = makeregularend(tend.boxes[tend.boxn - 1], BOTTOM,
                          ND_coord(tn).y - GD_rank(g)[ND_rank(tn)].ht1);
@@ -1908,8 +1875,8 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
       P->start.theta = -M_PI / 2, P->start.constrained = true;
       smode = false;
     }
-    boxes_append(&boxes, rank_box(sp, g, ND_rank(tn)));
-    b = hend.nb = maximal_bbox(g, sp, hn, e, NULL);
+    LIST_APPEND(&boxes, rank_box(sp, g, ND_rank(tn)));
+    b = hend.nb = maximal_bbox(g, *sp, hn, e, NULL);
     endpath(P, hackflag ? &fwdedgeb.out : e, REGULAREDGE, &hend,
             spline_merge(aghead(e)));
     b.UR.y = hend.boxes[hend.boxn - 1].UR.y;
@@ -1918,7 +1885,7 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
     if (b.LL.x < b.UR.x && b.LL.y < b.UR.y)
       hend.boxes[hend.boxn++] = b;
     completeregularpath(P, segfirst, e, &tend, &hend, &boxes);
-    boxes_free(&boxes);
+    LIST_FREE(&boxes);
     pointf *ps = NULL;
     size_t pn = 0;
     if (is_spline)
@@ -1936,12 +1903,12 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
     }
     if (pn == 0) {
       free(ps);
-      points_free(&pointfs);
-      points_free(&pointfs2);
+      LIST_FREE(&pointfs);
+      LIST_FREE(&pointfs2);
       return;
     }
     for (size_t i = 0; i < pn; i++) {
-      points_append(&pointfs, ps[i]);
+      LIST_APPEND(&pointfs, ps[i]);
     }
     free(ps);
     recover_slack(segfirst, P);
@@ -1951,39 +1918,37 @@ static void make_regular_edge(graph_t *g, spline_info_t *sp, path *P,
   /* make copies of the spline points, one per multi-edge */
 
   if (cnt == 1) {
-    points_sync(&pointfs);
-    clip_and_install(fe, hn, points_front(&pointfs), points_size(&pointfs),
-                     &sinfo);
-    points_free(&pointfs);
-    points_free(&pointfs2);
+    LIST_SYNC(&pointfs);
+    clip_and_install(fe, hn, LIST_FRONT(&pointfs), LIST_SIZE(&pointfs), &sinfo);
+    LIST_FREE(&pointfs);
+    LIST_FREE(&pointfs2);
     return;
   }
   const double dx = sp->Multisep * (cnt - 1) / 2;
-  for (size_t k = 1; k + 1 < points_size(&pointfs); k++)
-    points_at(&pointfs, k)->x -= dx;
+  for (size_t k = 1; k + 1 < LIST_SIZE(&pointfs); k++)
+    LIST_AT(&pointfs, k)->x -= dx;
 
-  for (size_t k = 0; k < points_size(&pointfs); k++)
-    points_append(&pointfs2, points_get(&pointfs, k));
-  points_sync(&pointfs2);
-  clip_and_install(fe, hn, points_front(&pointfs2), points_size(&pointfs2),
-                   &sinfo);
+  for (size_t k = 0; k < LIST_SIZE(&pointfs); k++)
+    LIST_APPEND(&pointfs2, LIST_GET(&pointfs, k));
+  LIST_SYNC(&pointfs2);
+  clip_and_install(fe, hn, LIST_FRONT(&pointfs2), LIST_SIZE(&pointfs2), &sinfo);
   for (unsigned j = 1; j < cnt; j++) {
     e = edges[ind + j];
     if (ED_tree_index(e) & BWDEDGE) {
-      MAKEFWDEDGE(&fwdedge.out, e);
+      makefwdedge(&fwdedge.out, e);
       e = &fwdedge.out;
     }
-    for (size_t k = 1; k + 1 < points_size(&pointfs); k++)
-      points_at(&pointfs, k)->x += sp->Multisep;
-    points_clear(&pointfs2);
-    for (size_t k = 0; k < points_size(&pointfs); k++)
-      points_append(&pointfs2, points_get(&pointfs, k));
-    points_sync(&pointfs2);
-    clip_and_install(e, aghead(e), points_front(&pointfs2),
-                     points_size(&pointfs2), &sinfo);
+    for (size_t k = 1; k + 1 < LIST_SIZE(&pointfs); k++)
+      LIST_AT(&pointfs, k)->x += sp->Multisep;
+    LIST_CLEAR(&pointfs2);
+    for (size_t k = 0; k < LIST_SIZE(&pointfs); k++)
+      LIST_APPEND(&pointfs2, LIST_GET(&pointfs, k));
+    LIST_SYNC(&pointfs2);
+    clip_and_install(e, aghead(e), LIST_FRONT(&pointfs2), LIST_SIZE(&pointfs2),
+                     &sinfo);
   }
-  points_free(&pointfs);
-  points_free(&pointfs2);
+  LIST_FREE(&pointfs);
+  LIST_FREE(&pointfs2);
 }
 
 /* regular edges */
@@ -2016,16 +1981,15 @@ static void completeregularpath(path *P, edge_t *first, edge_t *last,
   for (int i = 0; i < tendp->boxn; i++)
     add_box(P, tendp->boxes[i]);
   const size_t fb = P->nbox + 1;
-  const size_t lb = fb + boxes_size(boxes) - 3;
-  for (size_t i = 0; i < boxes_size(boxes); i++)
-    add_box(P, boxes_get(boxes, i));
+  const size_t lb = fb + LIST_SIZE(boxes) - 3;
+  for (size_t i = 0; i < LIST_SIZE(boxes); i++)
+    add_box(P, LIST_GET(boxes, i));
   for (int i = hendp->boxn - 1; i >= 0; i--)
     add_box(P, hendp->boxes[i]);
   adjustregularpath(P, fb, lb);
 }
 
-/* makeregularend:
- * Add box to fill between node and interrank space. Needed because
+/* Add box to fill between node and interrank space. Needed because
  * nodes in a given rank can differ in height.
  * for now, regular edges always go from top to bottom
  */
@@ -2037,8 +2001,7 @@ static boxf makeregularend(boxf b, int side, double y) {
   return (boxf){{b.LL.x, b.UR.y}, {b.UR.x, y}};
 }
 
-/* adjustregularpath:
- * make sure the path is wide enough.
+/* make sure the path is wide enough.
  * the % 2 was so that in rank boxes would only be grown if
  * they were == 0 while inter-rank boxes could be stretched to a min
  * width.
@@ -2088,13 +2051,10 @@ static void adjustregularpath(path *P, size_t fb, size_t lb) {
 }
 
 static boxf rank_box(spline_info_t *sp, graph_t *g, int r) {
-  boxf b;
-  node_t *left0, *left1;
-
-  b = sp->Rank_box[r];
+  boxf b = sp->Rank_box[r];
   if (b.LL.x == b.UR.x) {
-    left0 = GD_rank(g)[r].v[0];
-    left1 = GD_rank(g)[r + 1].v[0];
+    node_t *const left0 = GD_rank(g)[r].v[0];
+    node_t *const left1 = GD_rank(g)[r + 1].v[0];
     b.LL.x = sp->LeftBound;
     b.LL.y = ND_coord(left1).y + GD_rank(g)[r + 1].ht2;
     b.UR.x = sp->RightBound;
@@ -2128,9 +2088,9 @@ static edge_t *straight_path(edge_t *e, int cnt, points_t *plist) {
 
   while (cnt--)
     f = ND_out(aghead(f)).list[0];
-  assert(!points_is_empty(plist));
-  points_append(plist, points_get(plist, points_size(plist) - 1));
-  points_append(plist, points_get(plist, points_size(plist) - 1));
+  assert(!LIST_IS_EMPTY(plist));
+  LIST_APPEND(plist, LIST_GET(plist, LIST_SIZE(plist) - 1));
+  LIST_APPEND(plist, LIST_GET(plist, LIST_SIZE(plist) - 1));
 
   return f;
 }
@@ -2240,8 +2200,7 @@ static Agraph_t *cl_bound(graph_t *g, node_t *n, node_t *adj) {
   return rv;
 }
 
-/* maximal_bbox:
- * Return an initial bounding box to be used for building the
+/* Return an initial bounding box to be used for building the
  * beginning or ending of the path of boxes.
  * Height reflects height of tallest node on rank.
  * The extra space provided by FUDGE allows begin/endpath to create a box
@@ -2250,8 +2209,8 @@ static Agraph_t *cl_bound(graph_t *g, node_t *n, node_t *adj) {
  */
 #define FUDGE 4
 
-static boxf maximal_bbox(graph_t *g, spline_info_t *sp, node_t *vn, edge_t *ie,
-                         edge_t *oe) {
+static boxf maximal_bbox(graph_t *g, const spline_info_t sp, node_t *vn,
+                         edge_t *ie, edge_t *oe) {
   double b, nb;
   graph_t *left_cl, *right_cl;
   node_t *left, *right;
@@ -2263,19 +2222,19 @@ static boxf maximal_bbox(graph_t *g, spline_info_t *sp, node_t *vn, edge_t *ie,
   b = (double)(ND_coord(vn).x - ND_lw(vn) - FUDGE);
   if ((left = neighbor(g, vn, ie, oe, -1))) {
     if ((left_cl = cl_bound(g, vn, left)))
-      nb = GD_bb(left_cl).UR.x + sp->Splinesep;
+      nb = GD_bb(left_cl).UR.x + sp.Splinesep;
     else {
       nb = (double)(ND_coord(left).x + ND_mval(left));
       if (ND_node_type(left) == NORMAL)
         nb += GD_nodesep(g) / 2.;
       else
-        nb += sp->Splinesep;
+        nb += sp.Splinesep;
     }
     if (nb < b)
       b = nb;
     rv.LL.x = round(b);
   } else
-    rv.LL.x = fmin(round(b), sp->LeftBound);
+    rv.LL.x = fmin(round(b), sp.LeftBound);
 
   /* we have to leave room for our own label! */
   if (ND_node_type(vn) == VIRTUAL && ND_label(vn))
@@ -2284,19 +2243,19 @@ static boxf maximal_bbox(graph_t *g, spline_info_t *sp, node_t *vn, edge_t *ie,
     b = (double)(ND_coord(vn).x + ND_rw(vn) + FUDGE);
   if ((right = neighbor(g, vn, ie, oe, 1))) {
     if ((right_cl = cl_bound(g, vn, right)))
-      nb = GD_bb(right_cl).LL.x - sp->Splinesep;
+      nb = GD_bb(right_cl).LL.x - sp.Splinesep;
     else {
       nb = ND_coord(right).x - ND_lw(right);
       if (ND_node_type(right) == NORMAL)
         nb -= GD_nodesep(g) / 2.;
       else
-        nb -= sp->Splinesep;
+        nb -= sp.Splinesep;
     }
     if (nb > b)
       b = nb;
     rv.UR.x = round(b);
   } else
-    rv.UR.x = fmax(round(b), sp->RightBound);
+    rv.UR.x = fmax(round(b), sp.RightBound);
 
   if (ND_node_type(vn) == VIRTUAL && ND_label(vn)) {
     rv.UR.x -= ND_rw(vn);
