@@ -372,8 +372,7 @@ static graph_t *mkConstraintG(Dt_t * list, intersectfn intersect, distfn dist) {
 
 static void closeGraph(graph_t * cg)
 {
-    node_t *n;
-    for (n = agfstnode(cg); n; n = agnxtnode(cg, n)) {
+    for (node_t *n = agfstnode(cg); n; n = agnxtnode(cg, n)) {
 	free_list(ND_in(n));
 	free_list(ND_out(n));
     }
@@ -677,14 +676,12 @@ static double compress(info * nl, int nn)
     return sc;
 }
 
-DEFINE_LIST(points, pointf)
-
 static pointf *mkOverlapSet(info *nl, size_t nn, size_t *cntp) {
     info *p = nl;
     info *q;
-    points_t S = {0};
+    LIST(pointf) S = {0};
 
-    points_append(&S, (pointf){0});
+    LIST_APPEND(&S, (pointf){0});
 
     for (size_t i = 0; i < nn; i++) {
 	q = p + 1;
@@ -705,16 +702,17 @@ static pointf *mkOverlapSet(info *nl, size_t nn, size_t *cntp) {
 		    if (pt.y < 1)
 			pt.y = 1;
 		}
-		points_append(&S, pt);
+		LIST_APPEND(&S, pt);
 	    }
 	    q++;
 	}
 	p++;
     }
 
-    points_shrink_to_fit(&S);
-    *cntp = points_size(&S);
-    return points_detach(&S);
+    LIST_SHRINK_TO_FIT(&S);
+    pointf *ret;
+    LIST_DETACH(&S, &ret, cntp);
+    return ret;
 }
 
 static pointf computeScaleXY(pointf *aarr, size_t m) {
