@@ -11,6 +11,7 @@
 #include "config.h"
 
 #include <stdbool.h>
+#include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -54,6 +55,16 @@ typedef struct {
     int idx;
 } GVertex;
 
+/// convert a `GtsVertex` pointer to a pointer to its derived class
+static GVertex *downcast(GtsVertex *base) {
+  return (GVertex *)((char *)base - offsetof(GVertex, v));
+}
+
+/// convert a `GVertex` pointer to a pointer to its base class
+static GtsVertex *upcast(GVertex *derived) {
+  return &derived->v;
+}
+
 typedef struct {
     GtsVertexClass parent_class;
 } GVertexClass;
@@ -143,14 +154,14 @@ tri(double *x, double *y, int npt, int *segs, int nsegs, int sepArr)
 
     if (sepArr) {
 	for (i = 0; i < npt; i++) {
-	    GVertex *p = (GVertex *) gts_vertex_new(vcl, x[i], y[i], 0);
+	    GVertex *p = downcast(gts_vertex_new(vcl, x[i], y[i], 0));
 	    p->idx = i;
 	    vertices[i] = p;
 	}
     }
     else {
 	for (i = 0; i < npt; i++) {
-	    GVertex *p = (GVertex *) gts_vertex_new(vcl, x[2*i], x[2*i+1], 0);
+	    GVertex *p = downcast(gts_vertex_new(vcl, x[2*i], x[2*i+1], 0));
 	    p->idx = i;
 	    vertices[i] = p;
 	}
@@ -162,8 +173,8 @@ tri(double *x, double *y, int npt, int *segs, int nsegs, int sepArr)
      */
     for (i = 0; i < nsegs; i++) {
 	edges[i] = gts_edge_new(ecl,
-		 (GtsVertex *)vertices[segs[2 * i]],
-		 (GtsVertex *)vertices[segs[2 * i + 1]]);
+		 upcast(vertices[segs[2 * i]]),
+		 upcast(vertices[segs[2 * i + 1]]));
     }
 
     for (i = 0; i < npt; i++)
@@ -179,7 +190,7 @@ tri(double *x, double *y, int npt, int *segs, int nsegs, int sepArr)
 					       t->e1, t->e2, t->e3));
 
     for (i = 0; i < npt; i++) {
-	GtsVertex *v4 = (GtsVertex *)vertices[i];
+	GtsVertex *v4 = upcast(vertices[i]);
 	GtsVertex *v = gts_delaunay_add_vertex(surface, v4, NULL);
 
 	/* if v != NULL, it is a previously added pt with the same
@@ -222,8 +233,8 @@ static int cnt_edge(void *edge, void *stats) {
 
     sp->n++;
     if (sp->delaunay) {
-	sp->delaunay[((GVertex*)e->v1)->idx].nedges++;
-	sp->delaunay[((GVertex*)e->v2)->idx].nedges++;
+	sp->delaunay[downcast(e->v1)->idx].nedges++;
+	sp->delaunay[downcast(e->v2)->idx].nedges++;
     }
 
     return 0;
@@ -239,8 +250,8 @@ static int add_edge(void *edge, void *data) {
     GtsSegment *e = edge;
     v_data *delaunay = data;
 
-    int source = ((GVertex*)e->v1)->idx;
-    int dest = ((GVertex*)e->v2)->idx;
+    int source = downcast(e->v1)->idx;
+    int dest = downcast(e->v2)->idx;
 
     delaunay[source].edges[delaunay[source].nedges++] = dest;
     delaunay[dest].edges[delaunay[dest].nedges++] = source;
@@ -290,8 +301,8 @@ static int addEdge(void *edge, void *state) {
     GtsSegment *e = edge;
     estate *es = state;
 
-    int source = ((GVertex*)e->v1)->idx;
-    int dest = ((GVertex*)e->v2)->idx;
+    int source = downcast(e->v1)->idx;
+    int dest = downcast(e->v2)->idx;
 
     es->edges[2 * es->n] = source;
     es->edges[2 * es->n + 1] = dest;
@@ -416,9 +427,9 @@ static int addFace(void *face, void *state) {
     GtsVertex *v1, *v2, *v3;
 
     gts_triangle_vertices (&f->v.triangle, &v1, &v2, &v3);
-    *ip++ = ((GVertex*)v1)->idx;
-    *ip++ = ((GVertex*)v2)->idx;
-    *ip++ = ((GVertex*)v3)->idx;
+    *ip++ = downcast(v1)->idx;
+    *ip++ = downcast(v2)->idx;
+    *ip++ = downcast(v3)->idx;
 
     ni.nneigh = 0;
     ni.neigh = neigh;
@@ -438,9 +449,9 @@ static int addTri(void *face, void *state) {
     GtsVertex *v1, *v2, *v3;
 
     gts_triangle_vertices (&f->v.triangle, &v1, &v2, &v3);
-    *ip++ = ((GVertex*)v1)->idx;
-    *ip++ = ((GVertex*)v2)->idx;
-    *ip++ = ((GVertex*)v3)->idx;
+    *ip++ = downcast(v1)->idx;
+    *ip++ = downcast(v2)->idx;
+    *ip++ = downcast(v3)->idx;
 
     return 0;
 }
@@ -531,167 +542,6 @@ freeSurface (surface_t* s)
     free (s->neigh);
     free(s);
 }
-#elif defined(HAVE_TRIANGLE)
-#define TRILIBRARY
-#include <triangle.c>
-#include <assert.h>
-#include <sparse/general.h>
-
-int*
-get_triangles (double *x, int n, int* tris)
-{
-    struct triangulateio mid, vorout;
-    int i;
-
-    if (n <= 2) return NULL;
-
-    struct triangulateio in = {.numberofpoints = n};
-    in.pointlist = gv_calloc(in.numberofpoints * 2, sizeof(REAL));
-
-    for (i = 0; i < n; i++){
-	in.pointlist[i*2] = x[i*2];
-	in.pointlist[i*2 + 1] = x[i*2 + 1];
-    }
-    mid.pointlist = NULL; // Not needed if -N switch used.
-    mid.pointattributelist = NULL;
-    mid.pointmarkerlist = NULL; /* Not needed if -N or -B switch used. */
-    mid.trianglelist = NULL; // Not needed if -E switch used.
-    mid.triangleattributelist = NULL;
-    mid.neighborlist = NULL; // Needed only if -n switch used.
-    mid.segmentlist = NULL;
-    mid.segmentmarkerlist = NULL;
-    mid.edgelist = NULL; // Needed only if -e switch used.
-    mid.edgemarkerlist = NULL; // Needed if -e used and -B not used.
-    vorout.pointlist = NULL; // Needed only if -v switch used.
-    vorout.pointattributelist = NULL;
-    vorout.edgelist = NULL; // Needed only if -v switch used.
-    vorout.normlist = NULL; // Needed only if -v switch used.
-
-    /* Triangulate the points.  Switches are chosen to read and write a  */
-    /*   PSLG (p), preserve the convex hull (c), number everything from  */
-    /*   zero (z), assign a regional attribute to each element (A), and  */
-    /*   produce an edge list (e), a Voronoi diagram (v), and a triangle */
-    /*   neighbor list (n).                                              */
-
-    triangulate("Qenv", &in, &mid, &vorout);
-    assert (mid.numberofcorners == 3);
-
-    *tris = mid.numberoftriangles;
-    
-    free(in.pointlist);
-    free(in.pointattributelist);
-    free(in.pointmarkerlist);
-    free(in.regionlist);
-    free(mid.pointlist);
-    free(mid.pointattributelist);
-    free(mid.pointmarkerlist);
-    free(mid.triangleattributelist);
-    free(mid.neighborlist);
-    free(mid.segmentlist);
-    free(mid.segmentmarkerlist);
-    free(mid.edgelist);
-    free(mid.edgemarkerlist);
-    free(vorout.pointlist);
-    free(vorout.pointattributelist);
-    free(vorout.edgelist);
-    free(vorout.normlist);
-
-    return mid.trianglelist;
-}
-
-// maybe it should be replaced by RNG - relative neighborhood graph, or by GG - gabriel graph
-int* 
-delaunay_tri (double *x, double *y, int n, int* nedges)
-{
-    int i;
-
-    struct triangulateio in = {
-	.numberofpoints = n,
-	.pointlist = gv_calloc(2 * n, sizeof(REAL))
-    };
-    for (i = 0; i < n; i++) {
-	in.pointlist[2 * i] = x[i];
-	in.pointlist[2 * i + 1] = y[i];
-    }
-
-    struct triangleio out = {.numberofpoints = n};
-
-    triangulate("zQNEeB", &in, &out, NULL);
-
-    *nedges = out.numberofedges;
-    free (in.pointlist);
-    free (in.pointattributelist);
-    free (in.pointmarkerlist);
-    free (in.trianglearealist);
-    free (in.triangleattributelist);
-    free (in.neighborlist);
-    free (in.segmentlist);
-    free (in.segmentmarkerlist);
-    free (in.holelist);
-    free (in.regionlist);
-    free (in.edgemarkerlist);
-    free (in.normlist);
-    free (out.pointattributelist);
-    free (out.pointmarkerlist);
-    free (out.trianglearealist);
-    free (out.triangleattributelist);
-    free (out.neighborlist);
-    free (out.segmentlist);
-    free (out.segmentmarkerlist);
-    free (out.holelist);
-    free (out.regionlist);
-    free (out.edgemarkerlist);
-    free (out.normlist);
-    return out.edgelist;
-}
-
-static v_data *delaunay_triangulation(double *x, double *y, int n) {
-    int nedges;
-    int source, dest;
-    int* edgelist = delaunay_tri (x, y, n, &nedges);
-    int i;
-
-    v_data *delaunay = gv_calloc(n, sizeof(v_data));
-    int *edges = gv_calloc(2 * nedges + n, sizeof(int));
-
-    for (i = 0; i < n; i++) {
-	delaunay[i].ewgts = NULL;
-	delaunay[i].nedges = 1;
-    }
-
-    for (i = 0; i < 2 * nedges; i++)
-	delaunay[edgelist[i]].nedges++;
-
-    for (i = 0; i < n; i++) {
-	delaunay[i].edges = edges;
-	edges += delaunay[i].nedges;
-	delaunay[i].edges[0] = i;
-	delaunay[i].nedges = 1;
-    }
-    for (i = 0; i < nedges; i++) {
-	source = edgelist[2 * i];
-	dest = edgelist[2 * i + 1];
-	delaunay[source].edges[delaunay[source].nedges++] = dest;
-	delaunay[dest].edges[delaunay[dest].nedges++] = source;
-    }
-
-    free(edgelist);
-    return delaunay;
-}
-
-surface_t* 
-mkSurface (double *x, double *y, int n, int* segs, int nsegs)
-{
-    agerrorf("mkSurface not yet implemented using Triangle library\n");
-    assert (0);
-    return 0;
-}
-void 
-freeSurface (surface_t* s)
-{
-    agerrorf("freeSurface not yet implemented using Triangle library\n");
-    assert (0);
-}
 #else
 static char* err = "Graphviz built without any triangulation library\n";
 int* get_triangles (double *x, int n, int* tris)
